unit IirZeros;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ToolWin, ExtCtrls, TeeProcs, TeEngine, Chart,
  SignalToolsTee, TeeEdit, SignalToolsDialogs, SpectrumAna, MtxBaseComp,
  SignalTools, SignalAnalysis, Series, StdCtrls, TeeTools, FileSignal,
  MtxDialogs, MtxComCtrls, TeePolar;

type
  TIirZerosForm = class(TForm)
    ZeroPoleChart: TSpectrumChart;
    Panel1: TPanel;
    ToolBar1: TToolBar;
    ChartEditButton: TToolButton;
    ChartEditor: TChartEditor;
    RichEdit1: TRichEdit;
    ChartTool1: TAxisScaleTool;
    OrderEdit: TMtxFloatEdit;
    Label1: TLabel;
    Label2: TLabel;
    AttEdit: TMtxFloatEdit;
    FilterBox: TComboBox;
    Label3: TLabel;
    Series1: TPolarSeries;
    Series2: TPolarSeries;
    procedure ChartEditButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OrderEditChange(Sender: TObject);
    procedure FilterBoxChange(Sender: TObject);
  private
    procedure FillSeries(Order,Att: integer);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  IirZerosForm: TIirZerosForm;

implementation

{$R *.dfm}

uses SignalUtils, MtxVec, IIRFilters, MtxVecTee, Math387;

procedure TIirZerosForm.ChartEditButtonClick(Sender: TObject);
begin
    ChartEditor.Execute;
end;


procedure TIirZerosForm.FormCreate(Sender: TObject);
begin
     FilterBox.ItemIndex := 0;
     With RichEdit1.Lines, RichEdit1 do
     begin
        Clear;
        Add('DSP for MtxVec features 4 different filter design methods for infinite impulse response filters: ' +
            'Butterworth, Chebyshev type I and type II and elliptic filters. Change the order and attenuation ' +
            'in the stop band of a low-pass IIR filter designed with all 4 methods.');
        Paragraph.FirstIndent := 0;
        Paragraph.Numbering := nsNone;
     end;
     FillSeries(5,40);
end;


procedure TIirZerosForm.OrderEditChange(Sender: TObject);
begin
     FillSeries(OrderEdit.IntPosition,AttEdit.IntPosition);
end;

procedure TIirZerosForm.FillSeries(Order,Att: integer);
var z,p,r,phi: TVec;
    k: TSample;
begin
     CreateIt(z,p,r,phi);
     try
         case TIirFilterMethod(FilterBox.ItemIndex) of
         fimButter:      ButterFilter(Order,[0.2],ftLowPass,false,z,p,k);
         fimChebyshevI:  ChebyshevIFilter(Order,0.1,[0.2],ftLowPass,false,z,p,k);
         fimChebyshevII: ChebyshevIIFilter(Order,Att,[0.2],ftLowPass,false,z,p,k);
         fimElliptic:    EllipticFilter(Order,0.1,Att,[0.2],ftLowPass,false,z,p,k);
         end;
         if not z.Complex then z.ExtendToComplex(True);
         z.CartToPolar(r,phi);
         phi.Scale(180/pi);
         DrawValues(phi,r, zeroPoleChart.Series[0]);
         if not p.Complex then p.ExtendToComplex(True);
         p.CartToPolar(r,phi);
         phi.Scale(180/pi);         
         DrawValues(phi,r, zeroPoleChart.Series[1]);
     finally
         FreeIt(z,p,r,phi);
     end;
end;

procedure TIirZerosForm.FilterBoxChange(Sender: TObject);
begin
     FillSeries(OrderEdit.IntPosition,AttEdit.IntPosition);
end;

initialization
RegisterClass(TIirZerosForm);

end.
