{*******************************************}
{   Signal Tools 1.0 DSP Components	    }
{    					    }
{   Types: TMarkRecord                      }
{                                           }
{   Components: TSignalMarkTool             }
{               TSpectrumMarkTool           }
{               TAxisScaleTool              }
{               TSpectrumChart              }
{               TSignalChart                }
{                                           }
{   Copyright (c) 1997-2003 Dew Research    }
{                 Janez Makovsek            }
{   All Rights Reserved                     }
{*******************************************}
{$IMPORTEDDATA ON}
{$I BdsppDefs.inc}
{:Additional TeeTools and TChart components for TeeChart.}
unit SignalToolsTee;

interface

uses Windows,SysUtils,Chart, SignalTools, MtxVec, Math387,
     TeEngine, TeeTools, Controls, Graphics, SignalAnalysis,
     Classes,Series, TeCanvas, TeeConst, SignalSeriesTee;

type
      {:Defines which spectrum will be displayed on the chart.
        With sppAmplt the amplitude spectrum will drawn and with
        sppPhase the phase spectrum will be drawn on the TSpectrumChart.}
      TSpectrumPart = (sppAmplt, sppPhase);

      {:Abstract class for drawing signals and spectrums.
        Abstract class for drawing signals and spectrums.}
      TAnalysisChart = class(TChart)
      private    {Add SaveToStream and LoadFromStream}
        FPixelDownSample: boolean;
        FAutoAnalysisUpdate: boolean;
        procedure SetPixelDownSample(const Value: boolean);
        procedure SetAutoAnalysisUpdate(const Value: boolean);
      protected
        procedure InternalDrawValues(Index: integer); virtual;
        function  InputIndex(Input: TAnalysis; var Index: integer): boolean; virtual; abstract;
        procedure InputData(Index: integer; var a: TVec;var Offset, Step: TSample); virtual; abstract;
        {Offset: bandwidthL, or Time = 0, Step is Dt or HzRes}
        function  InputCount: integer; virtual; abstract;
        procedure NotifyChartUpdate(Sender: TOBject);
      public
        {::DrawValues will add all data of the TSignal/TSpectrum component located
           at Index in the TSignal/TSpectrum list to the series located at Index in
           the Chart.SeriesList.}
        procedure DrawValues(Index: Integer); virtual;
        {::Create the component.}
        constructor Create(AOwner: TComponent); override;
        {::Destroy the component.}
        destructor Destroy; override;
      published
        property BevelOuter default bvLowered;
        property Color default clWhite;
        property View3D default False;
        {::If True, the series associated with TSignal/TSpectrum component will be
           redrawn each time TAnalysis.Update method is called.}
        property  AutoAnalysisUpdate: boolean read FAutoAnalysisUpdate write SetAutoAnalysisUpdate default True;
        {::Set this property to True to enable pixel-downsampling. Set TVec.PixelDownSample
           for more info.}
        property  PixelDownSample: boolean read FPixelDownSample write SetPixelDownSample default false;
      end;

      {:TChart descendant supports drawing of data from connected TSignal components. 
        This TChart component allows you to connect (associate) TSignal descendants
         with chart series from the Chart.SeriesList .}
      TSignalChart = class(TAnalysisChart)
      private
        FSignals: TSignalCollection;
        procedure SetSignals(const Value: TSignalCollection);
      protected
        procedure InternalDrawValues(Index: integer); override;
        function  InputIndex(Input: TAnalysis; var Index: integer): boolean; override;
        procedure InputData(Index: integer; var a: TVec;var Offset, Step: TSample); override;
        function  InputCount: integer; override;
        procedure Notification(AComponent: TComponent; Operation: TOperation); override;
      public
        {::Create the component.}
        constructor Create(AOwner: TComponent); override;
        {::Destroy the component.}
        destructor Destroy; override;
      published
        {::Maintaines a list of pointers to TSignal objects.}
        property  Signals: TSignalCollection read FSignals write SetSignals;
      end;

     {:TChart descendant supports drawing of data from connected TSpectrum components.
       This TChart component allows you to link (associate) TSpectrum descendants
       with chart series from the Chart.SeriesList. }
      TSpectrumChart = Class(TAnalysisChart)
      private
        FSpectrumPart: TSpectrumPart;
        FSpectrums: TSpectrumCollection;
        FOnAfterMarkPeak: TNotifyEvent;
        FOnSpectrumMode: TNotifyEvent;
        procedure SetSpectrumMode(const Value: TSpectrumPart);
        procedure SetSpectrums(const Value: TSpectrumCollection);
        procedure SetOnAfterMarkPeak(const Value: TNotifyEvent);
        procedure SetOnSpectrumMode(const Value: TNotifyEvent);
      protected
        procedure InternalDrawValues(Index: integer); override;
        function  InputIndex(Input: TAnalysis; var Index: integer): boolean; override;
        {:Returns data associate with Index.}
        procedure InputData(Index: integer; var a: TVec;var Offset, Step: TSample); override;
        function  InputCount: integer; override;
        procedure Notification(AComponent: TComponent; Operation: TOperation); override;
      public
        {::Create the component.}
        constructor Create(AOwner: TComponent); override;
        {::Destroy the component.}
        destructor Destroy; override;
      published
        property Color default clWhite;
        {::Defines which part of the spectrum will be displayed, magnitude or phase.
          When this value is changed, all connected TSpectrums will be update the
          their associated series.}
        property SpectrumPart: TSpectrumPart read FSpectrumPart write SetSpectrumMode default sppAmplt;
        {::A collection of pointers to TSpectrum objects. The index in the Spectrums collection
           matches the index of the series. When the Update method of TSpectrum is called, it
           will also update the associated series, if it is connected to the Spectrums collection.}
        property Spectrums: TSpectrumCollection read FSpectrums write SetSpectrums;
        {::Triggered after a peak has been marked. This event will be called only by
           either TSpectrumMarkTool or TSignalMarkTool.}
        property OnAfterMarkPeak: TNotifyEvent read FOnAfterMarkPeak write SetOnAfterMarkPeak;
        {::Triggered after the spectrumMode property has been changed and all series
           have been updated. }
        property OnSpectrumMode: TNotifyEvent read FOnSpectrumMode write SetOnSpectrumMode;
      end;

      {:Waterfall chart.
        Does nothing yet.}
      TWaterfallChart = class(TSpectrumChart)
      private
        FWaterfallSize: integer;
        procedure SetWaterfallSize(const Value: integer);
      protected
        procedure InternalDrawValues(Index: integer); override;
      published
        property WaterfallSize: integer read FWaterfallSize write SetWaterfallSize;
      end;


    {Provide means to:

     - Draw cursor values on the chart (Enabled, Disabled)
     - Add or delete marked peaks from attached series
     - Needs to know spectrum index. (Channel index)

     Probably will need separate tool for time signal.}

    {:An abstract class provides basics for peak and signal marking.
      An abstract class provides basics for peak marking feature
      for a frequency spectrum and measurement cursor feature for
      a time signal.}
    TCursorMarkTool = class(TTeeCustomToolSeries)
    private
       OldLength: integer;
       TempPeaks: TMarkList;
       FDisplayCoordinates: boolean;
       fCursorActive: boolean;
       FMarkSeries: TChartSeries;
       FDoubleClickClear: boolean;
       procedure SetDisplayCoordinates(const Value: boolean);
       Procedure CalcValuePositions(X,Y:Integer);
       procedure CalcScreenPositions;
       procedure SetCursorActive(const Value: boolean);
       procedure SetDoubleClickClear(const Value: boolean);
    protected
       MouseShiftState: TShiftState;
       MouseY, MouseX: integer;
       CurrentMark: TMarkRecord;

       IXValue : Double;  { 5.01 }
       IYValue : Double;
       IPoint  : TPoint;
       property DisplayCoordinates: boolean read FDisplayCoordinates write SetDisplayCoordinates;
       procedure SetMarkSeries(const Value: TChartSeries); virtual;
       procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
       procedure MouseMove(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual; abstract;
       procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual; abstract;
       procedure GetMarkText(Sender: TChartSeries; ValueIndex: Integer; var MarkText: String); virtual; abstract;
       procedure AdjustTitle;
       procedure ResetTitle;
       procedure DrawCoordinates;
       Procedure ChartMouseEvent( AEvent: TChartMouseEvent;
                                  Button:TMouseButton;
                                  Shift: TShiftState; X, Y: Integer); override;
    public
       {::Create the component.}
       Constructor Create(AOwner:TComponent); override;
       {::Destroy the component.}
       destructor Destroy; override;
    published
       {::Set this property to false, to disable this TeeTool and hide the cursor.}
       property CursorActive: boolean read FCursorActive write SetCursorActive;
       {::Defines a chart series to draw marks. Usually this is a TPointSeries
         or some other series usefull to make marked points stand out.
         TChart only provides means to draw all marks or every second, third etc...
         By drawing marks on a separate series, you can have any number of marks
         at any location ranging from zero to values count.}
       property MarkSeries: TChartSeries read FMarkSeries write SetMarkSeries;
       {::If True, all marked points will deleted when the user double clicks the
          chart.}
       property DoubleClickClear: boolean read FDoubleClickClear write SetDoubleClickClear;
    end;


    {:Defines how the peaks will be marked.
      Defines how the peaks will be marked, when the user moves the mouse or clicks with the left
       mouse button. mmSingle: marks only the nearest peak, mmHarmoic: marks a harmonic series.
       mmSideBand: requires that you first mark the modulation frequency and then
                   the right sideband. The tool will additionaly mark the left
                   sideband and the base frequency.
       mmModes: requires that you first mark the first mode (the frequency to be modulated)
                and then the second mode (the modulating frequency). The tool will
                additionally mark both sidebands.
       mmtFreeStyle: Unlike the four other marking modes, this one will allow the user
                to mark any value, even, if it is not a peak.}
    TSpectrumMarkMode = (mmSingle, mmHarmonic, mmSideBand, mmModes, mmFreeStyle);
    {:Specifies labels displayed above the marked peak.
     Specifies what will the labels above the marked peak display. mtAmplt: only
       one value denoting amplitude, mtPhase: only one value denoting phase
       and with mtAmpltPhase both amplitude and phase will be displayed.}
    TSpectrumMarkType = (mtAmplt, mtPhase, mtAmpltPhase);

    {:Marks peaks in the frequency spectrums.
      Use this Tool to examine peaks (their exact frequency, amplitude and phase)
       of a frequency spectrum.}
    TSpectrumMarkTool = class(TCursorMarkTool)
    private
      FirstMark: TSample;
      FMarkMode: TSpectrumMarkMode;
      FMarkType: TSpectrumMarkType;
      FPeakFilterMode: boolean;
      FAmpltFormat: string;
      FPhaseFormat: string;
      FFreqFormat: string;
      FLabelHeaders: boolean;
      FPeakZoomBand: boolean;
      procedure SetMarkMode(const Value: TSpectrumMarkMode);
      procedure SetMarkType(const Value: TSpectrumMarkType);
      procedure SetPeakFilterMode(const Value: boolean);
      procedure SetAmpltFormat(const Value: string);
      procedure SetFreqFormat(const Value: string);
      procedure SetPhaseFormat(const Value: string);
      procedure SetLabelHeaders(const Value: boolean);
      procedure SetPeakZoomBand(const Value: boolean);
      procedure AddMarkXY(var a: TMarkRecord; const ALabel: string);
    protected
      procedure GetMarkText(Sender: TChartSeries; ValueIndex: Integer; var MarkText: String); override;
      procedure MouseMove(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      Procedure ChartEvent(AEvent:TChartToolEvent); override;
      procedure DrawMarks(MSpectrum: TSpectrum); virtual;
      procedure AddMark(MSpectrum: TSpectrum; Freq: TSample); virtual;
      procedure AddPeak(MSpectrum: TSpectrum; Toggle: boolean; x, y: integer); virtual;
      class Function GetEditorClass:String; override;
      procedure SetParentChart(const Value: TCustomAxisPanel); override;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
      function GetSpectrum: TSpectrum;
    public
      {::Returns a description of the tool. Used by TChart}
      class Function Description:String; override;
      {::Clears a list of existing marks.}
      procedure ClearMarks;
      {::Filters all marked peaks and clears a list of existing marks.}
      procedure FilterPeaks;
      {::Create the component.}
      Constructor Create(AOwner:TComponent); override;
      {::Destroy the component.}
      destructor Destroy; override;
    published
      {::If this property is True, the tool be enabled and will interact with the TeeChart.}
      property Active;
      {::Assign a TChartSeries to the Series property. This series must
         must be associated with a TSpectrumAnalyzer via TSpectrumChart.Spectrums property.}
      property Series;
      {::Set this property to True, if you want to use digital filter to remove
         each clicked peak, instead to mark it.}
      property PeakFilterMode: boolean read FPeakFilterMode write SetPeakFilterMode;
      {::Defines the way marks will be marked.}
      property MarkMode: TSpectrumMarkMode read FMarkMode write SetMarkMode;
      {::Specifies marks type.}
      property MarkType: TSpectrumMarkType read FMarkType write SetMarkType;
      {::Defines the number format (like: 0.0##) used to format the amplitude
         value displayed for the marked peak.}
      property AmpltFormat: string read FAmpltFormat write SetAmpltFormat;
      {::Defines the number format (like: 0.0##) used to format the phase
         value displayed for the marked peak.}
      property PhaseFormat: string read FPhaseFormat write SetPhaseFormat;
      {::Defines the number format (like: 0.0##) used to format the frequency
         value displayed for the marked peak.}
      property FreqFormat: string read FFreqFormat write SetFreqFormat;
      {::If True, the label displaying the amplitude of the marked peak, will
         be preceded with 'A = ' and the phase will be preceded with 'P = '.}
      property LabelHeaders: boolean read FLabelHeaders write SetLabelHeaders;
      {::If True and if the frequency spectrum is zoomed, the peak search
         will be restricted only to the displayed bandwidth.}
      property PeakZoomBand: boolean read FPeakZoomBand write SetPeakZoomBand;
    end;

    {:Defines which time series values to find.
      Defines which signal values will be marked, when the user moves the mouse or clicks with the left
       mouse button.
       tmMaximum: marks only the nearest maximum,
       tmMinimum: marks only the nearest minimum,
       tmZeroCrossing: marks only the nearest zero crossing by using linear interpolation.
       tmFreeStyle: marks the nearest series value.}

    TSignalMarkMode = (tmMaximum, tmMinimum, tmZeroCrossing, tmFreeStyle);

    {:Mark specific signal values.
      Use this Tool to examine time series: minimums, maximums amd zero crossings.}
    TSignalMarkTool = class(TCursorMarkTool)
    private
      mSecs: boolean;
      FAmpltFormat: string;
      FTimeFormat: string;
      FMarkMode: TSignalMarkMode;
      procedure SetAmpltFormat(const Value: string);
      procedure SetTimeFormat(const Value: string);
      procedure DrawMarks(MSignal: TSignal);
      procedure SetMarkMode(const Value: TSignalMarkMode);
      procedure AddMark(MSignal: TSignal; Time: double);
      procedure AddPeak(MSignal: TSignal; Toggle: boolean; x, y: integer);
      procedure AddMarkXY(var a: TMarkRecord; const ALabel: string);
    protected
      procedure GetMarkText(Sender: TChartSeries; ValueIndex: Integer; var MarkText: String); override;
      procedure MouseMove(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
      procedure SetParentChart(const Value: TCustomAxisPanel); override;
      Procedure ChartEvent(AEvent:TChartToolEvent); override;
      class Function GetEditorClass:String; override;
      function GetSignal: TSignal;
    public
      {::Create the component.}
      Constructor Create(AOwner:TComponent); override;
      {::Returns a description of the tool. Used by TChart}
      class Function Description:String; override;
      {::Clear all marks.}
      procedure ClearMarks;
    published
      property Active;
      property Series;
      {::Defines the number format (like: 0.0##) used to format the amplitude
         value displayed for the marked peak.}
      property AmpltFormat: string read FAmpltFormat write SetAmpltFormat;
      {::Defines the number format (like: 0.0##) used to format the time
         value displayed for the marked peak.}
      property TimeFormat: string read FTimeFormat write SetTimeFormat;
      {::Defines how the marks will be drawn.}
      property MarkMode: TSignalMarkMode read FMarkMode write SetMarkMode;
    end;

    {:Specifies the axis scaling.
      Axis scaling can be:
      @amNone The default axis scaling of the TeeChart will not be affected.
      @amPeakHold The maximum value of the consecutive chart updates will be
                  used as the maximum for all subsequent updates.
      @amAuto     The axis will be scaled automatically, but it is be possible
                  to define top and bottom margins in
                  percentage of the total axis span.}
    TAxisScaleMode = (amNone, amPeakHold, amAuto);

    {:TeeTool extends the scaling capabalities of the axes on a TChart.
      Use this component add more options to the default axis scaling
      provided by TeeChart. The tool allows you to define top and bottom
      margins for any axis and a "peak hold" behaviour.}
    TAxisScaleTool = class(TTeeCustomToolAxis)
    private
      Logarithmic: boolean;
      OldMax: double;
      OldMin: double;
      FAxisScaleMode: TAxisScaleMode;
      FUpperMargin: integer;
      FLowerMargin: integer;
      FDataIsSpectrum: boolean;
      procedure SetAxisScaleMode(const Value: TAxisScaleMode);
      procedure SetLowerMargin(const Value: integer);
      procedure SetUpperMargin(const Value: integer);
      procedure SetDataIsSpectrum(const Value: boolean);
    protected
      Procedure ChartEvent(AEvent:TChartToolEvent); override;
      class Function GetEditorClass:String; override;
    public
      class function Description: String; override;
      Constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
      procedure Reset; {reset peak hold}
    published
      property Active;
      property Axis;
      {::Defines the upper margin on the axis as a percentage of the total
        span of the values.}
      property UpperMargin: integer read FUpperMargin write SetUpperMargin;
      {::Defines the lower margin on the axis as a percentage of the total
        span of the values.}
      property LowerMargin: integer read FLowerMargin write SetLowerMargin;
      {:Defines how this TeeTool will affect the scaling of the axis.}
      property AxisScaleMode: TAxisScaleMode read FAxisScaleMode write SetAxisScaleMode;
      {:If TAxisScaleTool is used as on the left axis of the frequency spectrum,
        checking this property will reset the axis PeakHold scaling when switching
        between linear and logarithmic scales.}
      property DataIsSpectrum: boolean read FDataIsSpectrum write SetDataIsSpectrum;
    end;

  {:Assign a TChart to TChart.
    Assigns the state of the Source chart (all series and property values)
    to the Destination chart. This routine makes use of the ResetProperties
    routine, which ensures that all properties will get assigned.}
  procedure AssignChart(Source, Destination: TChart); overload;

implementation

uses MtxVecTee, TeeStore, MtxBaseComp, SignalUtils,
     SpectrumMarkToolEdit,SignalMarkToolEdit, AxisScaleToolEdit;

 const TeeMsg_SpectrumMarkTool  = 'Spectrum mark cursor';
       TeeMsg_SignalMarkTool  = 'Signal mark cursor';
       TeeMsg_AxisScaleTool = 'Axis scale';

procedure AssignChart(Source, Destination: TChart);
var a: TMemoryStream;
    balign: TAlign;
    Top,Left,Width,Height: integer;
begin
    a := TMemoryStream.Create;
    try
        balign := Destination.Align;
        Top := Destination.Top;
        Left := Destination.Left;
        Width := Destination.Width;
        Height := Destination.Height;
//        MtxSaveChartToStream(Source,a);
        SaveChartToStream(Source,a);
        a.Position := 0;
        ResetProperties(TCustomChart(Destination));
        LoadChartFromStream(TCustomChart(Destination),a);
//        MtxSaveChartToStream(Source,a);        
        Destination.Align := balign;
        Destination.Top := Top;
        Destination.Left := Left;
        Destination.Width := Width;
        Destination.Height := Height;
    finally
        a.Destroy;
    end;
end;

{ TSpectrumChart }

constructor TSpectrumChart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSpectrums := TSpectrumCollection.Create(Self);
  FSpectrums.OnSetInput := NotifyChartUpdate;
  FSpectrumPart := sppAmplt;
end;

destructor TSpectrumChart.Destroy;
begin
  FreeAndNil(FSpectrums);
  inherited Destroy;
end;

function TSpectrumChart.InputCount: integer;
begin
      Result := Spectrums.Count;  
end;

procedure TSpectrumChart.InputData(Index: integer; var a: TVec;var Offset, Step: TSample);
var FSpectrum: TSpectrum;
begin
    FSpectrum := FSpectrums.Items[Index].Input;
    if Assigned(FSpectrum) then
    case FSpectrumPart of
    sppAmplt: a := FSpectrum.Amplt;
    sppPhase: a := FSpectrum.Phase;
    end;
    Step := FSpectrum.HzRes;
    Offset := FSpectrum.BandWidthL;
end;

function TSpectrumChart.InputIndex(Input: TAnalysis; var Index: integer): boolean;
var i: integer;
begin
     Result := False;
     for i := Index to Spectrums.Count-1 do
     begin
          if Spectrums.Items[i].Input = Input then
          begin
             Result := True;
             Index := i;
          end;
     end;
end;

procedure TSpectrumChart.InternalDrawValues(Index: integer);
var FSpectrum: TSpectrum;
begin     {You can switch to which series the data will be drawn by setting the
           Index property of the TCollectionItem or moving the series}
  if Index > (SeriesList.Count-1) then Exit;
  FSpectrum := FSpectrums.Items[Index].Input;
  if Assigned(FSpectrum) then
  case FSpectrumPart of
  sppAmplt: MtxVecTee.DrawValues(FSpectrum.Amplt,Series[Index], FSpectrum.BandWidthL,FSpectrum.HzRes, FPixelDownSample);
  sppPhase: MtxVecTee.DrawValues(FSpectrum.Phase,Series[Index], FSpectrum.BandWidthL,FSpectrum.HzRes, FPixelDownSample);
  end;
end;

procedure TSpectrumChart.Notification(AComponent: TComponent;
  Operation: TOperation);
var i: integer;
begin
   if (Operation = opRemove) and  (AComponent <> Self) and (AComponent <> nil) then
   begin
       if Assigned(Spectrums) then
       for i := 0 to  Spectrums.Count-1 do
       begin
           if Assigned(Spectrums[i].Input) and (AComponent=Spectrums[i].Input) then Spectrums[i].Reset;
       end;
   end;
   inherited;
end;

procedure TSpectrumChart.SetOnAfterMarkPeak(const Value: TNotifyEvent);
begin
  FOnAfterMarkPeak := Value;
end;

procedure TSpectrumChart.SetOnSpectrumMode(const Value: TNotifyEvent);
begin
  FOnSpectrumMode := Value;
end;

procedure TSpectrumChart.SetSpectrumMode(const Value: TSpectrumPart);
var i: integer;
    AValue: TSpectrumPart;
begin
  if(Value <> sppAmplt) and (Value <> sppPhase) then AValue := sppAmplt else AValue := Value;
  if AValue <> FSpectrumPart then
  begin
       FSpectrumPart := AValue;
       for i := 0 to FSpectrums.Count-1 do InternalDrawValues(i);  //update the chart.
       if Assigned(FOnSpectrumMode) then OnSpectrumMode(Self);
  end;
end;

procedure TSpectrumChart.SetSpectrums(const Value: TSpectrumCollection);
begin
     Spectrums.Assign(Value);
end;

{ TSignalChart }

procedure TSignalChart.SetSignals(const Value: TSignalCollection);
begin
     Signals.Assign(Value);
end;

procedure TSignalChart.InternalDrawValues(Index: integer);
var FSig: TSignal;
begin {You can switch to which series the data will be drawn by setting the
           Index property of the TCollectionItem or moving the series}
  if Index > SeriesList.Count-1 then Exit;
  FSig := FSignals.Items[Index].Input;
  MtxVecTee.DrawValues(FSig.Data,Series[Index], FSig.MinX,FSig.Dt, FPixelDownSample);
end;

constructor TSignalChart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSignals := TSignalCollection.Create(Self);
  FSignals.OnSetInput := NotifyChartUpdate;
end;

destructor TSignalChart.Destroy;
begin
  FreeAndNil(FSignals);
  inherited Destroy;
end;     

function TSignalChart.InputIndex(Input: TAnalysis; var Index: integer): boolean;
var i: integer;
begin
     Result := False;
     for i := Index to Signals.Count-1 do
     begin
          if Signals.Items[i].Input = Input then
          begin
             Result := True;
             Index := i;
          end;
     end;
end;

procedure TSignalChart.InputData(Index: integer; var a: TVec; var Offset,Step: TSample);
var sig: TSignal;
begin
    sig := Signals.Items[Index].Input;
    a := Sig.Data;
    Step := Sig.Dt;
    Offset := Sig.BandWidthL;
end;

procedure TSignalChart.Notification(AComponent: TComponent;
  Operation: TOperation);
var i: integer;
begin
   if (Operation = opRemove) and  (AComponent <> Self) and (AComponent <> nil) then
   begin
       if Assigned(Signals) then
       for i := 0 to  Signals.Count-1 do
       begin
           if Assigned(Signals[i].Input) and (AComponent=Signals[i].Input) then Signals[i].Reset;
       end;
   end;
   inherited;
end;


function TSignalChart.InputCount: integer;
begin
     Result := Signals.Count;
end;

{ TWaterfallChart }

procedure TWaterfallChart.InternalDrawValues;
begin
  inherited;

end;

procedure TWaterfallChart.SetWaterfallSize(const Value: integer);
begin
  FWaterfallSize := Value;
end;

{ TAnalysisChart }

constructor TAnalysisChart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoAnalysisUpdate := True;
  View3D := False;
  BevelOuter := bvLowered;
  Color := clWhite;
end;

destructor TAnalysisChart.Destroy;
begin
  inherited;
  //
end;
{
procedure TAnalysisChart.SaveToFile(FileName: string);
var AFile: TFileStream;
begin
     AFile := TFileStream.Create(FileName,fmCreate or fmOpenWrite);
     try
        SaveToStream(AFile);
     finally
     AFile.Destroy;
     end;
end;

procedure TAnalysisChart.LoadFromFile(FileName: string);
var AFile: TFileStream;
begin
     if not FileExists(FileName) then Exit;
     AFile := TFileStream.Create(FileName,fmOpenRead);
     try
        LoadFromStream(AFile);
     finally
     AFile.Destroy;
     end;
end;

procedure TAnalysisChart.LoadFromStream(Stream: TStream);
begin
    Stream.ReadComponent(Self);
end;

procedure TAnalysisChart.SaveToStream(Stream: TStream);
begin
    Stream.WriteComponent(Self);
end;}

procedure TAnalysisChart.DrawValues(Index: integer);
begin
     InternalDrawValues(Index);
end;

procedure TAnalysisChart.InternalDrawValues(Index: integer);
begin
   //
end;

procedure TAnalysisChart.NotifyChartUpdate(Sender: TOBject);
var i: integer;
begin
     if AutoAnalysisUpdate then
     begin
           i := 0; {Allows you to draw the same TAnalysis to more Series}
           while InputIndex(TAnalysis(Sender), i) do
           begin
               DrawValues(i);
               Inc(i);
           end;
     end;
end;

procedure TAnalysisChart.SetAutoAnalysisUpdate(const Value: boolean);
begin
  FAutoAnalysisUpdate := Value;
end;

procedure TAnalysisChart.SetPixelDownSample(const Value: boolean);
begin
  FPixelDownSample := Value;
end;

{ TCursorMarkTool}

constructor TCursorMarkTool.Create(AOwner: TComponent);
begin
  inherited Create(Owner);
  FDisplayCoordinates := True;
  FMarkSeries := nil;
  TempPeaks := TMarkList.Create;
  FCursorActive := True;
  FDoubleClickClear := True;
  IXValue:=0;
  IYValue:=0;
  IPoint.X:=-1;
  IPoint.Y:=-1;
  OldLength := 0;
end;

Procedure TCursorMarkTool.CalcScreenPositions;
begin
  if (IPoint.X=-1) or (IPoint.Y=-1) then
  begin
    With GetHorizAxis do IPoint.X:=(IStartPos+IEndPos) div 2;
    With GetVertAxis do IPoint.Y:=(IStartPos+IEndPos) div 2;
    CalcValuePositions(IPoint.X,IPoint.Y);
    CalcScreenPositions;
  end
  else
  begin
    IPoint.X:=GetHorizAxis.CalcPosValue(IXValue);
    IPoint.Y:=GetVertAxis.CalcPosValue(IYValue);
  end;
end;

Procedure TCursorMarkTool.CalcValuePositions(X,Y:Integer);
begin
  IXValue:= GetHorizAxis.CalcPosPoint(X);
  IYValue:= GetVertAxis.CalcPosPoint(Y);
end;

destructor TCursorMarkTool.Destroy;
begin
  TempPeaks.Destroy;
  inherited;
end;

procedure TCursorMarkTool.DrawCoordinates;
var  MyString: string;
begin
   if not DisplayCoordinates then Exit;
   with ParentChart do
   begin
       Canvas.Font.Color := BottomAxis.LabelsFont.Color;
       Canvas.Font.Size := BottomAxis.LabelsFont.Size - 1;
       Canvas.Font.Name := BottomAxis.LabelsFont.Name;
       Canvas.Brush.Color := Color;
       Canvas.Brush.Style := bsSolid;
       MyString := StringOfChar(' ', OldLength+10); {Erase old}
       Canvas.TextOut(ChartRect.Left ,ChartRect.Top + Canvas.Font.Height -8, MyString);
       MyString := FormatFloat(GetHorizAxis.AxisValuesFormat, IXValue);
       MyString := 'X: ' + MyString + ' Y: ';
       MYString :=  MyString + FormatFloat(GetVertAxis.AxisValuesFormat, IYValue);
       OldLength := Length(MyString);
       Canvas.TextOut(ChartRect.Left ,ChartRect.Top + Canvas.Font.Height -8, MyString);
   end;
end;

procedure TCursorMarkTool.SetCursorActive(const Value: boolean);
begin
     SetBooleanProperty(FCursorActive,Value);
end;

procedure TCursorMarkTool.SetDisplayCoordinates(const Value: boolean);
begin
     FDisplayCoordinates := Value;
end;

Procedure TCursorMarkTool.ChartMouseEvent( AEvent: TChartMouseEvent;
                                         Button:TMouseButton;
                                         Shift: TShiftState; X, Y: Integer);

begin
  if not Assigned(MarkSeries) then Exit;
  if not Assigned(Series) then Exit;
  Case AEvent of
      cmeUp: MouseUp(Button, Shift, X, Y);
    cmeMove: MouseMove(Button, Shift, X, Y);
    cmeDown: MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TCursorMarkTool.SetDoubleClickClear(const Value: boolean);
begin
  FDoubleClickClear := Value;
end;

procedure TCursorMarkTool.SetMarkSeries(const Value: TChartSeries);
begin
  if FMarkSeries <> nil then
  begin
      FMarkSeries.OnGetMarkText := nil;
      FMarkSeries := nil;
  end;
  if (Value <> FMarkSeries) then
  begin
      FMarkSeries := Value;
      if Assigned(FMarkSeries) then FMarkSeries.OnGetMarkText := GetMarkText;
      if Assigned(FMarkSeries) then FMarkSeries.FreeNotification(Self);
  end;
end;

procedure TCursorMarkTool.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
     MouseX := X;  {Remember the down position, and then compare with up position}
     MouseY := Y;  {If the same, then add peak otherwise do not}
     MouseShiftState := Shift;
end;

procedure TCursorMarkTool.AdjustTitle;
var MyChar: string;
begin
    MyChar := Series.GetHorizAxis.Title.Caption;
    if (Length(MyChar) > 0) and (Series.GetHorizAxis.LabelStyle = talAuto) then
    begin
      if MyChar[1] <> TeeLineSeparator then
      begin
        Series.GetHorizAxis.Title.Caption := TeeLineSeparator + MyChar;
        Series.GetHorizAxis.TitleSize := -Round((Series.GetHorizAxis.Title.Font.Height)*2.5);
      end;
    end;
end;

procedure TCursorMarkTool.ResetTitle;
var MyChar: string;
begin
    MyChar := Series.GetHorizAxis.Title.Caption;
    if (Length(MyChar) > 0) and (Series.GetHorizAxis.LabelStyle = talAuto) then
    begin
        if MyChar[1] = TeeLineSeparator then
        begin
           Series.GetHorizAxis.Title.Caption := System.Copy(MyChar,2,Length(MyChar)-1);
           Series.GetHorizAxis.TitleSize := 0; {Reset}
        end;
    end;
end;

{TSpectrumMarkTool}

Constructor TSpectrumMarkTool.Create(AOwner:TComponent);
begin
  inherited Create(Owner);
  FAmpltFormat := '0.00#';
  FFreqFormat := '0.00#';
  FPhaseFormat := '0.00#';
  FLabelHeaders := True;
end;

class Function TSpectrumMarkTool.GetEditorClass:String;
begin
  result:= 'TSpectrumMarkToolEditor';
end;

class Function TSpectrumMarkTool.Description: String;
begin
  result:= TeeMsg_SpectrumMarkTool;
end;

procedure TSpectrumMarkTool.AddMarkXY(var a: TMarkRecord; const ALabel: string);
var SC: TSpectrumChart;
begin
   ClearFPU; //for Delphi 5
   if MarkSeries is TSignalDiscreteSeries then
   begin
        TSignalDiscreteSeries(MarkSeries).TargetZOrder := Series.ParentChart.MaxZOrder-Series.ZOrder;
   end;
   SC := TSpectrumChart(ParentChart);
   case SC.SpectrumPart of
   sppAmplt: begin
            if not IsNanInf(a.Freq) and not IsNanInf(a.Amplt) then
            MarkSeries.AddXY(a.Freq,a.Amplt,ALabel,clTeeColor);
            end;
   sppPhase: begin
            if not IsNanInf(a.Freq) and not IsNanInf(a.Phase) then
            MarkSeries.AddXY(a.Freq,a.Phase,ALabel,clTeeColor);
            end;
   end;
end;

procedure TSpectrumMarkTool.AddMark(MSpectrum: TSpectrum; Freq: TSample);
var  AlreadyExists: boolean;
     i: integer;
     CurrentMark: TMarkRecord;
     sa: TSpectrumAnalyzer;
     SC: TSpectrumChart;
begin
       SC := TSpectrumChart(ParentChart);
       CurrentMark.Freq := Freq;
       if CheckRange(MSpectrum.BandwidthL,CurrentMark.Freq,MSpectrum.BandwidthH) then
       begin
            if MSpectrum is TSpectrumAnalyzer then
            begin
                sa := TSpectrumAnalyzer(MSpectrum);
                CurrentMark.Amplt := sa.AmpltEst(Freq);
                if sa.Logarithmic then sa.AmpltLogEst(CurrentMark.Amplt,Freq);
            end else CurrentMark.Amplt := MSpectrum.AmpltEst(Freq);
            CurrentMark.Phase := MSpectrum.PhaseEst(Freq);
            AlreadyExists := False;
           {current peak is drawn separatelly to check for exsistence of an existing peak at the same spot}
            case SC.SpectrumPart of
            sppAmplt: begin
                     for i := 0 to MarkSeries.XValues.Count-1 do
                         if MarkSeries.XValues.Value[i] = CurrentMark.Amplt then AlreadyExists := True;
                     end;
            sppPhase: begin
                     for i := 0 to MarkSeries.XValues.Count-1 do
                         if MarkSeries.XValues.Value[i] = CurrentMark.Phase then AlreadyExists := True;
                     end;
            end;
            if not AlreadyExists then
            begin
                 AddMarkXY(CurrentMark,FormatFloat(FFreqFormat, CurrentMark.Freq));
                 TempPeaks.Add(CurrentMark); {But not to TMarkList}
            end;
       end;
end;

procedure TSpectrumMarkTool.DrawMarks(MSpectrum: TSpectrum);
var i: integer;
    MyChar: string;
    CurrentMark: TMarkRecord;
begin
    if not Assigned(MarkSeries) then Exit;
    if not Assigned(Series) then Exit;
    if not Assigned(MSpectrum) then Exit;
    MarkSeries.Clear; {Remove all points}
    TempPeaks.Clear;
    MSpectrum.Marks.SortFreq;
    for i := 0 to MSpectrum.Marks.Count-1 do
    begin  {Add all points sorted from left to right}
          CurrentMark := MSpectrum.Marks[i]^;
          if Odd(i) then MyChar := TeeLineSeparator else MyChar := '';
          AddMarkXY(CurrentMark , MyChar + FormatFloat(FFreqFormat, CurrentMark.Freq));
          {Previously marked peaks are added}
          TempPeaks.Add(CurrentMark);   {Containes currently drawn peaks
           (Which might Count more then Marks.Count after the mouse moves) }
          AdjustTitle;
    end;
    if MSpectrum.Marks.Count = 0 then ResetTitle;
end;

procedure TSpectrumMarkTool.MouseMove(Button:TMouseButton; Shift: TShiftState; X, Y: Integer);
var Myy, HzDetected, HzRes, Bwl: double;
    Harmonic, SecondMode, SideBand, FirstMode, LowerSideBand, UpperSideBand: double;
    TheMiddle: integer;
    MSpectrum: TSpectrum;
    i, Harmonics: integer;
begin
    if not Assigned(MarkSeries) then Exit;
    if not Assigned(Series) then Exit;
    if (ssLeft in Shift) or (ssRight in Shift) or (ssMiddle in Shift) then Exit;
    if CursorActive then
    begin
        MSpectrum := GetSpectrum;
        if MSpectrum = nil then Exit;
        if MSpectrum.Length = 0 then Exit;

        {Calculate bound in pixels}
        Series.GetCursorValues(HzDetected, MyY);
        Bwl := MSpectrum.BandwidthL;
        HzRes := MSpectrum.HzRes;
        TheMiddle := Trunc((HZDetected-Bwl)/HzRes);  {Currently selected spectral line}
        if THeMiddle <= 0 then TheMiddle := 1;
        NearestMaximum(MSpectrum.Amplt,TheMiddle);
        DrawMarks(MSpectrum);
        case MarkMode of
        mmFreeStyle: begin
                     FirstMark := -1;
                     CurrentMark.Freq := Trunc((HZDetected-Bwl)/HzRes)*HzRes+Bwl;
                     AddMark(MSpectrum,CurrentMark.Freq);
                     end;
        mmSingle: begin
                  FirstMark := -1;
                  CurrentMark.Freq := MSpectrum.PeakApproximate(TheMiddle*HzRes+Bwl); {TheMiddle containes the peak}
                  AddMark(MSpectrum,CurrentMark.Freq);
                  end;
        mmHarmonic: {more then one peak is drawn}
                  begin
                  {Make a list of peaks to be added}
                  FirstMark := -1;
                  FirstMode := MSpectrum.PeakApproximate(TheMiddle*HzRes+Bwl);
                  Harmonics := MSpectrum.Harmonics;
                  for i := 0 to Harmonics-1 do
                  begin
                        Harmonic := FirstMode*(i+1);
                        AddMark(MSpectrum,Harmonic);
                  end;
                  CurrentMark.Freq := FirstMode;
                  end;
        mmSideband:         {Mark at first the second mode and then one of the sidebands}
                  begin
                       if (FirstMark = -1) then
                       begin
                            CurrentMark.Freq := MSpectrum.PeakApproximate(TheMiddle*HzRes+Bwl);
                            AddMark(MSpectrum,CurrentMark.Freq);
                       end else
                       begin    
                            CurrentMark.Freq := MSpectrum.PeakApproximate(TheMiddle*HzRes+Bwl);
                            AddMark(MSpectrum,CurrentMark.Freq); {add the first sideband}
                            SecondMode := FirstMark; {The first is the secoond mode peak}
                            SideBand := SecondMode + (SecondMode-CurrentMark.Freq);
                            AddMark(MSpectrum,SideBand);
                            FirstMode := abs(SecondMode-CurrentMark.Freq);
                            AddMark(MSpectrum,FirstMode);
                            CurrentMark.Freq := TheMiddle*HzRes+Bwl;
                       end;
                  end;
        mmModes:  begin {Mark the first mode and then the second}
                       if (FirstMark = -1) then
                       begin
                            CurrentMark.Freq := MSpectrum.PeakApproximate(TheMiddle*HzRes+Bwl);
                            AddMark(MSpectrum,CurrentMark.Freq);
                       end else
                       begin
                            CurrentMark.Freq := MSpectrum.PeakApproximate(TheMiddle*HzRes+Bwl);
                            AddMark(MSpectrum,CurrentMark.Freq);  {SecondMode}
                            FirstMode := FirstMark; {The first is the secoond mode peak, (use the first mark)}
                            SecondMode := MSpectrum.PeakApproximate(CurrentMark.Freq);
                            LowerSideBand := SecondMode - FirstMode;
                            AddMark(MSpectrum,LowerSideBand);
                            UpperSideBand := SecondMode + FirstMode;
                            AddMark(MSpectrum,UpperSideBand);
                            CurrentMark.Freq := TheMiddle*HzRes+Bwl;
                       end;
                  end;
        end;
        TempPeaks.SortFreq;
     end else
     begin
        CalcValuePositions(X,Y);
        CalcScreenPositions;
//        DrawCoordinates; {inherited MouseMove(Shift, X, Y)}
     end;
end;

procedure TSpectrumMarkTool.GetMarkText(Sender: TChartSeries; ValueIndex : Longint ; Var MarkText : String);
var TempMark: TMarkRecord;
    a: string;
     SC: TSpectrumChart;
begin
     a := '';
     if ValueIndex > TempPeaks.Count-1 then Exit;
     TempMark := TempPeaks[ValueIndex]^;
     SC := TSpectrumChart(ParentChart);
     case SC.SpectrumPart of
     sppAmplt:
         case MarkType of
         mtAmplt:      MarkText :=  FormatSample(FAmpltFormat, TempMark.Amplt);
         mtPhase:      begin
                          if FLabelHeaders then a := 'P=';
                          MarkText := a+FormatFloat(FPhaseFormat, TempMark.Phase);
                       end;
         mtAmpltPhase: begin
                          if fLabelHeaders then a := 'A=';
                          MarkText := a+FormatSample(FAmpltFormat, TempMark.Amplt)+ TeeLineSeparator;
                          if fLabelHeaders then a := 'P=';
                          MarkText := MarkText + a + FormatFloat(FPhaseFormat, TempMark.Phase);
                        end;
         end;
     sppPhase: MarkText := FormatFloat(FPhaseFormat, TempMark.Phase);
     end;
end;

procedure TSpectrumMarkTool.AddPeak(MSpectrum: TSpectrum; Toggle: boolean; x,y: integer);
var  ListPosition, i, Index: integer;
     TempMark: TMarkRecord;
     AlreadyExists: boolean;
     MyChar: string;
     sa: TSpectrumAnalyzer;
begin   {if exists then do not add, else add or delete}
     AlreadyExists := False;
     ListPosition := -1;
     for i := 0 to MSpectrum.Marks.Count-1 do
     begin
           if MSpectrum.Marks[i].Freq = CurrentMark.Freq then
           begin
                AlreadyExists := True;
                ListPosition := i;
           end;
     end;
     if not AlreadyExists then
     begin  {Add peak}
           if CheckRange(MSpectrum.BandwidthL,CurrentMark.Freq,MSpectrum.BandwidthH) then
           begin
                if MSpectrum is TSpectrumAnalyzer then
                begin
                    sa := TSpectrumAnalyzer(MSpectrum);
                    CurrentMark.Amplt := sa.AmpltEst(CurrentMark.Freq);
                    if sa.Logarithmic then sa.AmpltLogEst(CurrentMark.Amplt,CurrentMark.Freq);
                end else CurrentMark.Amplt := MSpectrum.AmpltEst(CurrentMark.Freq);
                CurrentMark.Phase := MSpectrum.PhaseEst(CurrentMark.Freq);
                Index := MSpectrum.Marks.Add(CurrentMark);
                if PeakFilterMode then
                begin
                     MSpectrum.FilterPeak(Index);
                     MSpectrum.Marks.Delete(Index);
                     MSpectrum.Update;
                end;
                DrawMarks(MSpectrum);
           end;
     end
     else
     begin {remove Peak}
           If Toggle then
           begin
               MSpectrum.Marks.Delete(ListPosition);
               MarkSeries.Clear;
               MSpectrum.Marks.SortFreq;
               for i:= 0 to MSpectrum.Marks.Count-1 do
               begin
                     TempMark := MSpectrum.Marks[i]^;
                     if Odd(i) then MyChar := TeeLineSeparator else MyChar := '';
                     AddMarkXY(TempMark,MyChar + FormatFloat(FFreqFormat, TempMark.Freq));
               end;
           end;
     end;
end;

function TSpectrumMarkTool.GetSpectrum: TSpectrum;
var SC: TSpectrumChart;
     i: integer;
begin
     SC := TSpectrumChart(ParentChart);
     i := SC.SeriesList.IndexOf(Series); {Dangerous for large amount of series}
     if (i < 0) or (i > (SC.Spectrums.Count-1)) then Result := nil else Result := SC.Spectrums[i].Input;
end;

procedure TSpectrumMarkTool.FilterPeaks;
var i: integer;
    MSPectrum: TSpectrum;
begin
    MSpectrum := GetSpectrum;
    if MSpectrum = nil then Exit;
    if MSpectrum.Length = 0 then Exit;
    ClearMarks;
    for i := 0 to MSpectrum.Marks.Count-1 do MSpectrum.FilterPeak(i);
    MSpectrum.Update;
end;

procedure TSpectrumMarkTool.ClearMarks;
var  MSPectrum: TSpectrum;
begin
    MSpectrum := GetSpectrum;
    if Assigned(MSpectrum) then MSpectrum.Marks.Clear;
    if Assigned(MarkSeries) then MarkSeries.Clear;
    TempPeaks.Clear;
    ResetTitle;
    FirstMark := -1;
end;


procedure TSpectrumMarkTool.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var  Harmonics, i: integer;
     Harmonic, SecondMode, SideBand, FirstMode, LowerSideBand, UpperSideBand: double;
     MSPectrum: TSpectrum;
begin
     if not ((MouseX = X) and (MouseY = Y)) then Exit;
     if CursorActive and (Button = mbLeft) and not (ssDouble in MouseShiftState) then
     begin
          MSpectrum := GetSpectrum;
          if MSpectrum = nil then Exit;
          if MSpectrum.Length = 0 then Exit;
          case MarkMode of
            mmSingle, mmFreeStyle:  AddPeak(MSpectrum,True, x,y);
            mmHarmonic: {more then one peak is drawn}
                      begin
                      {Make a list of peaks to be added}
                      FirstMode := MSpectrum.PeakApproximate(CurrentMark.Freq);
                      Harmonics := MSpectrum.Harmonics;
                      for i:= 0 to Harmonics-1 do
                      begin
                            Harmonic := FirstMode*(i+1);
                            CurrentMark.Freq := Harmonic;
                            AddPeak(MSpectrum,False,x,y);
                      end;
                      end;
            mmSideband:
                      begin
                           if (FirstMark = -1) then
                           begin
                                CurrentMark.Freq := MSpectrum.PeakApproximate(CurrentMark.Freq);
                                FirstMark := CurrentMark.Freq;
                                AddPeak(MSpectrum,False, x,y);
                           end else
                           begin
                                CurrentMark.Freq := MSpectrum.PeakApproximate(CurrentMark.Freq);
                                AddPeak(MSpectrum,False, x,y); {add the first sideband}
                                SecondMode := FirstMark; {The first is the secoond mode peak}
                                SideBand := SecondMode + (SecondMode-CurrentMark.Freq);
                                CurrentMark.Freq := SideBand;
                                AddPeak(MSpectrum,False, x,y);
                                FirstMode := abs(SecondMode-CurrentMark.Freq);
                                CurrentMark.Freq := FirstMode;
                                AddPeak(MSpectrum,False, x,y);
                                FirstMark := -1;
                           end;
                      end;
            mmModes:  begin
                           if (FirstMark = -1) then
                           begin
                                CurrentMark.Freq := MSpectrum.PeakApproximate(CurrentMark.Freq);
                                FirstMark := CurrentMark.Freq;
                                AddPeak(MSpectrum,false, x,y);
                           end else
                           begin
                                CurrentMark.Freq := MSpectrum.PeakApproximate(CurrentMark.Freq);
                                AddPeak(MSpectrum,False, x,y);
                                FirstMode := FirstMark; {The first is the secoond mode peak}
                                SecondMode := MSpectrum.PeakApproximate(CurrentMark.Freq);
                                LowerSideBand := SecondMode - FirstMode;
                                CurrentMark.Freq := LowerSideBand;
                                AddPeak(MSpectrum,False, x,y);
                                UpperSideBand := SecondMode + FirstMode;
                                CurrentMark.Freq := UpperSideBand;
                                AddPeak(MSpectrum,False, x,y);
                                FirstMark := -1; {Reset the first mark}
                           end;
                      end;
          end;
          if Assigned(TSpectrumChart(ParentChart).OnAfterMarkPeak) and (FirstMark = -1) then
                      TSpectrumChart(ParentChart).OnAfterMarkPeak(Self);
     end else
     if CursorActive and (Button = mbLeft) and (ssDouble in MouseShiftState)
                     and (FDoubleClickClear) then
                     begin
                          ClearMarks;
                          if Assigned(TSpectrumChart(ParentChart).OnAfterMarkPeak) then
                                      TSpectrumChart(ParentChart).OnAfterMarkPeak(Self);
                     end;
end;


procedure TSpectrumMarkTool.ChartEvent(AEvent: TChartToolEvent);
var MSpectrum: TSpectrumAnalyzer;
begin
  inherited;
  MSpectrum := TSpectrumAnalyzer(GetSpectrum);
  if not Assigned(MSpectrum) then Exit;
  if MSpectrum.Length = 0 then Exit;  
  if (AEvent=cteBeforeDraw) then
  begin
       if CursorActive then MouseMove(mbLeft, [],0,0) else DrawMarks(MSpectrum);
  end;
  if (AEvent = cteAfterDraw) then
  begin
       if PeakZoomBand then
       begin
           MSpectrum.Peaks.BandwidthH := Series.GetHorizAxis.Maximum;
           MSpectrum.Peaks.BandwidthL := Series.GetHorizAxis.Minimum;
       end;
  end;
end;

procedure TSpectrumMarkTool.SetParentChart(const Value: TCustomAxisPanel);
begin
  if (Value <> nil) then
  begin
      if not (Value is TSpectrumChart) then raise Exception.Create('TSpectrumChart type required!');
  end;
  inherited SetParentChart(Value);
end;

procedure TSpectrumMarkTool.Notification(AComponent: TComponent;Operation: TOperation);
begin
  inherited;
  if Operation=opRemove then
     if Assigned(FMarkSeries) and (AComponent=FMarkSeries) then
     begin
          if MarkSeries is TSignalDiscreteSeries then TSignalDiscreteSeries(MarkSeries).TargetZOrder := -1;
          FMarkSeries:=nil;
     end;
end;

destructor TSpectrumMarkTool.Destroy;
begin
  if Assigned(FMarkSeries) then FMarkSeries.OnGetMarkText := nil;
  inherited Destroy;
end;

procedure TSpectrumMarkTool.SetMarkMode(const Value: TSpectrumMarkMode);
begin
  FMarkMode := Value;
end;

procedure TSpectrumMarkTool.SetMarkType(const Value: TSpectrumMarkType);
begin
  FMarkType := Value;
end;

procedure TSpectrumMarkTool.SetPeakFilterMode(const Value: boolean);
begin
  FPeakFilterMode := Value;
end;

procedure TSpectrumMarkTool.SetAmpltFormat(const Value: string);
begin
  FAmpltFormat := Value;
end;

procedure TSpectrumMarkTool.SetFreqFormat(const Value: string);
begin
  FFreqFormat := Value;
end;

procedure TSpectrumMarkTool.SetPhaseFormat(const Value: string);
begin
  FPhaseFormat := Value;
end;

procedure TSpectrumMarkTool.SetLabelHeaders(const Value: boolean);
begin
  FLabelHeaders := Value;
end;

procedure TSpectrumMarkTool.SetPeakZoomBand(const Value: boolean);
begin
  FPeakZoomBand := Value;
end;

{ TAxisScaleTool }

procedure TAxisScaleTool.ChartEvent(AEvent: TChartToolEvent);
var Delta,Min,Max: double;
    tempAutoMin, tempAutoMax: boolean;
    i: integer;
    atool: TSpectrumMarkTool;
    alog: boolean;
    MSpectrum: TSpectrumAnalyzer;
begin
   inherited;
   if (AEvent=cteBeforeDraw) and Assigned(Axis) and not ParentChart.Zoomed then
   begin
        alog := false;
        if DataIsSpectrum then
        with ParentChart do
        begin  //Make sure that associated mark series is updated or Axis.CalcMinMax will not work right.}
            for i:=0 to Tools.Count-1 do
             if Tools[i].Active and (Tools[i] is TSpectrumMarkTool) then
             begin
                  atool := TSpectrumMarkTool(Tools[i]);
                  MSpectrum := TSpectrumAnalyzer(atool.GetSpectrum);
                  if Assigned(MSpectrum) then alog := MSpectrum.Logarithmic else alog := false;
                  if alog <> Logarithmic then atool.ChartEvent(cteBeforeDraw);
             end;
        end;
        if (AxisScaleMode <> amNone) and
           (not Axis.AutomaticMinimum or not Axis.AutomaticMaximum) then
        begin
             tempAutoMin := Axis.AutomaticMinimum;
             tempAutoMax := Axis.AutomaticMaximum;
             Axis.Automatic := True;
             Axis.CalcMinMax(Min,Max); {Because it checks for Auto}
             Axis.AutomaticMinimum := tempAutoMin;
             Axis.AutomaticMaximum := tempAutoMax;
             Delta := Max-Min;
             Max := UpperMargin*0.01*Delta + Max;
             if DataIsSpectrum and not alog then  Min := 0;
             Min := Min - LowerMargin*0.01*Delta;
        end else Reset;
        case AxisScaleMode of
        amNone: Reset;
        amPeakHold: begin
                        if DataIsSpectrum then
                        begin {Reset, if logarithmic was toggled.}
                           if alog <> Logarithmic then
                           begin
                               Logarithmic := alog;
                               Reset;
                           end;
                        end;
                        if (Min < OldMin) and not Axis.AutomaticMinimum then
                        begin
                             OldMin := Min;
                             Axis.Minimum := Min;
                        end;
                        if (Max > OldMax) and not Axis.AutomaticMaximum then
                        begin
                             OldMax := Max;
                             Axis.Maximum := Max;
                        end;
                    end;
        amAuto:     begin
                        Reset;
                        if not Axis.AutomaticMinimum then Axis.Minimum := Min;
                        if not Axis.AutomaticMaximum then Axis.Maximum := Max;
                    end;
        end;
   end;
end;

constructor TAxisScaleTool.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Reset;
end;

class function TAxisScaleTool.Description: String;
begin
      Result := TeeMsg_AxisScaleTool;
end;

destructor TAxisScaleTool.Destroy;
begin
  inherited;
end;

class function TAxisScaleTool.GetEditorClass: String;
begin
    Result := 'TAxisScaleToolEditor';
end;

procedure TAxisScaleTool.Reset;
begin
  OldMax := -MAXNUM;
  OldMin := MAXNUM;
end;

procedure TAxisScaleTool.SetAxisScaleMode(const Value: TAxisScaleMode);
begin
  FAxisScaleMode := Value;
end;

procedure TAxisScaleTool.SetDataIsSpectrum(const Value: boolean);
begin
  FDataIsSpectrum := Value;
end;

procedure TAxisScaleTool.SetLowerMargin(const Value: integer);
begin
  FLowerMargin := Value;
end;

procedure TAxisScaleTool.SetUpperMargin(const Value: integer);
begin
  FUpperMargin := Value;
end;

{ TSignalMarkTool }

procedure TSignalMarkTool.ClearMarks;
var MSignal: TSignal;
begin
    MSignal := GetSignal;
    if Assigned(MSignal) then MSignal.Marks.Clear;
    if Assigned(MarkSeries) then MarkSeries.Clear;
    TempPeaks.Clear;
    ResetTitle;
end;


procedure TSignalMarkTool.ChartEvent(AEvent: TChartToolEvent);
begin
  inherited;
  if (AEvent=cteBeforeDraw) then
  begin
       if CursorActive then MouseMove(mbLeft, [],0,0) else DrawMarks(GetSignal);
  end;
end;

constructor TSignalMarkTool.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAmpltFormat := '0.00#';
  FTimeFormat := '0.00#';
  mSecs := False;
end;

class function TSignalMarkTool.Description: String;
begin
  result:= TeeMsg_SignalMarkTool;
end;

function TSignalMarkTool.GetSignal: TSignal;
var SC: TSignalChart;
     i: integer;
begin
     SC := TSignalChart(ParentChart);
     i := SC.SeriesList.IndexOf(Series); {Dangerous for large amount of series}
     if (i < 0) or (i > (SC.Signals.Count-1)) then Result := nil else Result := SC.Signals[i].Input;
end;

procedure TSignalMarkTool.AddMarkXY(var a: TMarkRecord; const ALabel: string);
begin
     ClearFPU; //for Delphi 5 only
     if MarkSeries is TSignalDiscreteSeries then
     begin
          TSignalDiscreteSeries(MarkSeries).TargetZOrder := Series.ParentChart.MaxZOrder-Series.ZOrder;
     end;
     if not IsNanInf(a.Time) and not IsNanInf(a.Amplt) then
     MarkSeries.AddXY(a.Time,a.Amplt,ALabel,clTeeColor);
end;

procedure TSignalMarkTool.DrawMarks(MSignal: TSignal);
var i: integer;
    MyChar: string;
    CurrentMark: TMarkRecord;
begin
    if not Assigned(MarkSeries) then Exit;
    if not Assigned(Series) then Exit;
    if not Assigned(MSignal) then Exit;
    TempPeaks.Clear;
    MarkSeries.Clear;
    MSignal.Marks.SortTime;
    for i:= 0 to MSignal.Marks.Count-1 do
    begin
        CurrentMark := MSignal.Marks[i]^;
        if Odd(i) then MyChar := TeeLineSeparator else MyChar := '';
        if mSecs then CurrentMark.Time := CurrentMark.Time*1000;
        AddMarkXY(CurrentMark, MyChar + FormatFloat(FTimeFormat, CurrentMark.Time));
        TempPeaks.Add(CurrentMark);   {Containes currently drawn peaks}
        AdjustTitle;
    end;
    if MSignal.Marks.Count = 0 then ResetTitle;
end;

procedure TSignalMarkTool.AddPeak(MSignal: TSignal; Toggle: boolean; x,y: integer);
var  ListPosition, i: integer;
     TempMark: TMarkRecord;
     AlreadyExists: boolean;
     MyChar: string;
begin  {if exists then do not add else add or delete}
     ListPosition := 0;
     AlreadyExists := False;
     for i := 0 to  MSignal.Marks.Count-1 do
     begin
           if MSignal.Marks[i].Time = CurrentMark.Time then
           begin
                AlreadyExists := True;
                ListPosition := i;
           end;
     end;
     if not AlreadyExists then
     begin  {Add peak}
           if (CurrentMark.Time > 0) and (CurrentMark.Time < MSignal.SamplingTime) then
           begin
                MSignal.Marks.Add(CurrentMark);
                DrawMarks(MSignal);
           end;
     end
     else
     begin {remove Peak}
        If Toggle then
        begin
            MSignal.Marks.Delete(ListPosition);
            MarkSeries.Clear;
            MSignal.Marks.SortTime;
            for i := 0 to MSignal.Marks.Count-1 do
            begin
                 TempMark := MSignal.Marks[i]^;
                 if Odd(i) then MyChar := TeeLineSeparator else MyChar := '';
                 if mSecs then TempMark.Time := TempMark.Time*1000;
                 AddMarkXY(TempMark , MyChar + FormatFloat(fTimeFormat, TempMark.Time));
           end;
        end;
     end;
end;

procedure TSignalMarkTool.AddMark(MSignal: TSignal; Time: double);
var  AlreadyExists: boolean;
     i: integer;
begin
       ClearFPU;
       CurrentMark.Time := Time;
       if (CurrentMark.Time >= 0) and (CurrentMark.Time < MSignal.SamplingTime) then
       begin
            AlreadyExists := False;
            {current peak is drawn separatelly to check for exsistence of an existing peak at the same spot}
            for i := 0 to MarkSeries.XValues.Count-1 do
                 if MarkSeries.XValues.Value[i] = CurrentMark.Time then AlreadyExists := True;
            if not AlreadyExists then
            begin
                 if mSecs then MarkSeries.AddXY(CurrentMark.Time*1000,CurrentMark.Amplt ,FormatFloat(fTimeFormat, CurrentMark.Time*1000),clTeeColor) else
                 MarkSeries.AddXY(CurrentMark.Time,CurrentMark.Amplt ,FormatFloat(fTimeFormat, CurrentMark.Time),clTeeColor);
                 TempPeaks.Add(CurrentMark);
            end;
       end;
end;

procedure TSignalMarkTool.MouseMove(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var ATime,AAmplt: double;
    a,b: TSample;
    i: integer;
    MSignal: TSignal;
begin
      if not Assigned(MarkSeries) then Exit;
      if not Assigned(Series) then Exit;
      if (ssLeft in Shift) or (ssRight in Shift) or (ssMiddle in Shift) then Exit;
      if CursorActive then
      begin
            MSignal := GetSignal;
            if not Assigned(MSignal) then Exit;
            if MSignal.Length = 0 then Exit;

            Series.GetCursorValues(ATime, AAmplt);
            if ATIme < 0 then ATime := 0;
            if mSecs then ATime := ATime/1000;
            TempPeaks.Clear;
            DrawMarks(MSignal);
            i := Round(ATime/MSignal.Dt);
            case MarkMode of
            tmFreeStyle:    begin
                            CurrentMark.Time := i*MSignal.Dt;
                            CurrentMark.Amplt := MSignal.Data.Values[i];
                            AddMark(MSignal,CurrentMark.Time);
                            end;
            tmMaximum:      begin
                            NearestMaximum(MSignal.Data,i);
                            CurrentMark.Time := i*MSignal.Dt;
                            CurrentMark.Amplt := MSignal.Values[i];
                            AddMark(MSignal,CurrentMark.Time);
                            end;
            tmMinimum:      begin
                            NearestMinimum(MSignal.Data,i);
                            CurrentMark.Time := i*MSignal.Dt;
                            CurrentMark.Amplt := MSignal.Values[i];
                            AddMark(MSignal,CurrentMark.Time);
                            end;
            tmZeroCrossing: begin
                            a := i;
                            NearestZeroCrossing(MSignal.Data, a, b);
                            CurrentMark.Time := a*MSignal.Dt;
                            CurrentMark.Amplt := b;
                            AddMark(MSignal,CurrentMark.Time);
                            end;
            end;
            TempPeaks.SortTime;
      end else
      begin
            CalcValuePositions(X,Y);
            CalcScreenPositions;
//          DrawCoordinates;
      end;
end;

procedure TSignalMarkTool.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var MSignal: TSignal;
begin
     if not ((MouseX = X) and (MouseY = Y)) then Exit;
     if CursorActive and (Button = mbLeft) and not (ssDouble in MouseShiftState) then
     begin
          MSignal := GetSignal;
          if not Assigned(MSignal) then Exit;
          if MSignal.Length = 0 then Exit;
          AddPeak(MSignal,True,X,Y);
     end else
     if CursorActive and (Button = mbLeft) and (ssDouble in MouseShiftState)
                     and (FDoubleClickClear) then ClearMarks;
end;

procedure TSignalMarkTool.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation=opRemove then
     if Assigned(FMarkSeries) and (AComponent=FMarkSeries) then MarkSeries:=nil;
end;

procedure TSignalMarkTool.SetAmpltFormat(const Value: string);
begin
  FAmpltFormat := Value;
end;

procedure TSignalMarkTool.SetParentChart(const Value: TCustomAxisPanel);
begin
  if (Value <> nil) then
  begin
      if not (Value is TSignalChart) then ERaise('TSignalChart type required!');
  end;
  inherited SetParentChart(Value);
end;

procedure TSignalMarkTool.SetTimeFormat(const Value: string);
begin
  FTimeFormat := Value;
end;

procedure TSignalMarkTool.GetMarkText(Sender: TChartSeries;
  ValueIndex: Integer; var MarkText: String);
var TempMark: TMarkRecord;
begin
     if ValueIndex > TempPeaks.Count-1 then Exit;
     TempMark := TempPeaks[ValueIndex]^;
     MarkText := FormatSample(FAmpltFormat, TempMark.Amplt);
end;

class function TSignalMarkTool.GetEditorClass: String;
begin
   result:= 'TSignalMarkToolEditor';
end;

procedure TSignalMarkTool.SetMarkMode(const Value: TSignalMarkMode);
begin
  FMarkMode := Value;
end;

initialization
RegisterClass(TSpectrumChart);
RegisterClass(TSignalChart);
RegisterTeeTools([TSpectrumMarkTool, TSignalMarkTool, TAxisScaleTool]);

finalization
UnRegisterClass(TSpectrumChart);
UnRegisterClass(TSignalChart);
UnRegisterTeeTools([TSpectrumMarkTool, TSignalMarkTool, TAxisScaleTool]);

end.
